//////////////////////////////////////////////////////////////////////
//
// Routines to Convert NED-Body to NED-Inertial
//
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
//
// Direction Cosine Matrix routine
//
// DCM direction definition:
//    aNED = dcm * aBody
//    aBody = dcm.transpose() * aNED
//
// NED reference frames assumed for both body and inertial.
//
//////////////////////////////////////////////////////////////////////

void body2inertial() {

//#include "Eigen.h"
//#include <Eigen/Dense>

Eigen::Matrix<float,3,1> aBody = Eigen::Matrix<float,3,1>::Zero();
Eigen::Matrix<float,3,1>  aNED = Eigen::Matrix<float,3,1>::Zero();
Eigen::Matrix<float,3,3>   dcm = Eigen::Matrix<float,3,3>::Zero();

float rR, pR, yR;

// Load in accel and orientation angles
//const float gConstant = -9.80665;  // Gravity, m/s^2

/*
if(pR < 0){
  pR += 2*PI;
}
if(pR > 2*PI){
  pR -= 2*PI;
}
if(rR < 0){
  rR += 2*PI;
}
if(rR > 2*PI){
  rR -= 2*PI;
}
if(yR < 0){
  yR += 2*PI;
}
if(yR > 2*PI){
  yR -= 2*PI;
}
*/

/*
// These lines allow for testing of DCM direction  
pR =  0.0 * PI / 180.0;
//rR =  0.0 * PI / 180.0;
yR =  0.0 * PI / 180.0;
//yR = 45.0 * PI / 180.0;
aBody(0,0) = 0.0;
aBody(1,0) = 1.0;
aBody(2,0) = 0.0;
*/

/*
//Define DCM 
dcm(0,0) =  cosf(yR)* cosf(pR);
dcm(0,1) =  cosf(yR) * sinf(rR) * sinf(pR) - cosf(rR) * sinf(yR);
dcm(0,2) =  sinf(rR) * sinf(yR) + cosf(rR) * cosf(yR) * sinf(pR);

dcm(1,0) =  cosf(pR) * sinf(yR);
dcm(1,1) =  cosf(rR) * cosf(yR) + sinf(rR) * sinf(yR) * sinf(pR);
dcm(1,2) =  cosf(rR) * sinf(yR) * sinf(pR) - cosf(yR) * sinf(rR);

dcm(2,0) = -sinf(pR);
dcm(2,1) =  cosf(pR) * sinf(rR);
dcm(2,2) =  cosf(rR) * cosf(pR);
*/

aBody(0,0) = ax1;
aBody(1,0) = ay1;
aBody(2,0) = az1;

pR = ypr[1] * deg2rad;
rR = ypr[2] * deg2rad;
yR = ypr[0] * deg2rad;

//Define DCM 
dcm(0,0) =  cos(pR) * cos(yR);
dcm(0,1) =  cos(pR) * sin(yR);
dcm(0,2) = -sin(pR);

dcm(1,0) = -cos(rR) * sin(yR) + sin(rR) * sin(pR) * cos(yR) ;
dcm(1,1) =  cos(rR) * cos(yR) + sin(rR) * sin(pR) * sin(yR);
dcm(1,2) =  sin(rR) * cos(pR);

dcm(2,0) =  sin(rR) * sin(yR) + cos(rR) * sin(pR) * cos(yR);
dcm(2,1) = -sin(rR) * cos(yR) + cos(rR) * sin(pR) * sin(yR) ;
dcm(2,2) =  cos(rR) * cos(pR);

// Perform transformation
aNED = dcm.transpose() * aBody;

// Assign accel NED elements to global variables
aNEDx1 = aNED(0,0);
aNEDy1 = aNED(1,0);
aNEDz1 = aNED(2,0);

// Remove gravity from Z component
aNEDz1 += gravConst;

/*  // Test Code
Serial.print(pR), Serial.print(", ");
Serial.print(rR), Serial.print(", ");
Serial.print(yR), Serial.print(", ");
Serial.print(aBody(0,0)), Serial.print(", ");
Serial.print(aBody(1,0)), Serial.print(", ");
Serial.print(aBody(2,0)), Serial.print(", ");
Serial.print(aNED(0,0)), Serial.print(", ");
Serial.print(aNED(1,0)), Serial.print(", ");
Serial.println(aNED(2,0));

Serial.print(dcm(0,0)), Serial.print(", ");
Serial.print(dcm(0,1)), Serial.print(", ");
Serial.print(dcm(0,2)), Serial.print(", ");
Serial.print(dcm(1,0)), Serial.print(", ");
Serial.print(dcm(1,1)), Serial.print(", ");
Serial.print(dcm(1,2)), Serial.print(", ");
Serial.print(dcm(2,0)), Serial.print(", ");
Serial.print(dcm(2,1)), Serial.print(", ");
Serial.println(dcm(2,2));
*/

}

//////////////////////////////////////////////////////////////////////
//
// Quaternion Approach, NED-Body to NED-Inertial
//
//////////////////////////////////////////////////////////////////////

/*
void gravity_compensate(q, acc){

  g = {0.0f, 0.0f, 0.0f};

  // Get direction of gravity
  g(0) = 2 * (q(1) * q(3) - q(0) * q(2));
  g(1) = 2 * (q(0) * q(1) + q(2) * q(3));
  g(2) = q(0)*q(0) - q(1)*q(1) - q(2)*q(2) + q(3)*q(3);

  // Compensate accel readings with the direction of gravity
  aNEDXInertial(0) = acc(0) - g(0);
  aNEDXInertial(1) = acc(1) - g(1);
  aNEDXInertial(2) = acc(2) - g(2);
  
}
*/




